@@ -1,5 +1,5 @@
-#!/bin/env python
-""" Pyprland - an Hyprland companion app (cli client & daemon) """
+"""Pyprland - an Hyprland companion app (cli client & daemon)."""
+
 import asyncio
 import importlib
 import itertools
@@ -7,21 +7,16 @@ import json
 import os
 import sys
 import tomllib
+from collections.abc import Callable
 from functools import partial
-from typing import Any, Callable, Self
+from typing import Any, Self, cast
 
-from pyprland.common import (
-    IPC_FOLDER,
-    get_logger,
-    init_logger,
-    merge,
-    run_interactive_program,
-)
-from pyprland.ipc import get_event_stream
+from pyprland.common import IPC_FOLDER, get_logger, init_logger, merge, run_interactive_program
+from pyprland.ipc import get_event_stream, notify_error, notify_fatal, notify_info
 from pyprland.ipc import init as ipc_init
-from pyprland.ipc import notify_error, notify_fatal, notify_info
 from pyprland.plugins.interface import Plugin
 from pyprland.types import PyprError
+from pyprland.version import VERSION
 
 CONTROL = f"{IPC_FOLDER}/.pyprland.sock"
 OLD_CONFIG_FILE = "~/.config/hypr/pyprland.json"
@@ -33,7 +28,8 @@ __all__: list[str] = []
 
 
 class Pyprland:
-    "Main app object"
+    """Main app object."""
+
     server: asyncio.Server
     event_reader: asyncio.StreamReader
     stopped = False
@@ -44,27 +40,27 @@ class Pyprland:
     log_handler: Callable[[Plugin, str, tuple], None]
 
     @classmethod
-    def _set_instance(cls, instance):
-        "Set instance reference into class (for testing/debugging only)"
+    def _set_instance(cls, instance: Self) -> None:
+        """Set instance reference into class (for testing/debugging only)."""
         cls.instance = instance
 
-    def __init__(self):
+    def __init__(self) -> None:
         self.pyprland_mutex_event = asyncio.Event()
         self.pyprland_mutex_event.set()
         self.config = {}
         self.plugins: dict[str, Plugin] = {}
         self.log = get_logger()
-        self.queues = {}
+        self.queues: dict[str, asyncio.Queue] = {}
         self._set_instance(self)
 
-    async def initialize(self):
-        "Initializes the main structures"
+    async def initialize(self) -> None:
+        """Initialize the main structures."""
         await self.load_config()  # ensure sockets are connected first
 
-    async def __open_config(self, config_filename=""):
-        """Loads config file as self.config"""
+    async def __open_config(self, config_filename: str = "") -> dict[str, Any]:
+        """Load config file as self.config."""
         if config_filename:
-            fname = os.path.expandvars(os.path.expanduser(config_filename))
+            fname = os.path.expanduser(os.path.expandvars(config_filename))
             if os.path.isdir(fname):
                 config: dict[str, Any] = {}
                 for toml_file in sorted(os.listdir(fname)):
@@ -76,7 +72,7 @@ class Pyprland:
             if os.path.exists(OLD_CONFIG_FILE) and not os.path.exists(CONFIG_FILE):
                 self.log.warning("Consider changing your configuration to TOML format.")
             self.config.clear()
-            fname = os.path.expanduser(CONFIG_FILE)
+            fname = os.path.expanduser(os.path.expandvars(CONFIG_FILE))
 
         config = self.__load_config_file(fname)
 
@@ -86,8 +82,8 @@ class Pyprland:
             self.config.update(config)
         return config
 
-    def __load_config_file(self, fname):
-        """Loads a configuration file and returns it as a dictionary"""
+    def __load_config_file(self, fname: str) -> dict[str, Any]:
+        """Load a configuration file and returns it as a dictionary."""
         config = {}
         if os.path.exists(fname):
             self.log.info("Loading %s", fname)
@@ -96,18 +92,18 @@ class Pyprland:
                     config = tomllib.load(f)
                 except tomllib.TOMLDecodeError as e:
                     self.log.critical("Problem reading %s: %s", fname, e)
-                    raise PyprError() from e
+                    raise PyprError from e
         elif os.path.exists(os.path.expanduser(OLD_CONFIG_FILE)):
             self.log.info("Loading %s", OLD_CONFIG_FILE)
             with open(os.path.expanduser(OLD_CONFIG_FILE), encoding="utf-8") as f:
                 config = json.loads(f.read())
         else:
             self.log.critical("Config file not found! Please create %s", fname)
-            raise PyprError()
+            raise PyprError
         return config
 
-    async def _load_single_plugin(self, name, init) -> bool:
-        "Load a single plugin, optionally calling `init`"
+    async def _load_single_plugin(self, name: str, init: bool) -> bool:
+        """Load a single plugin, optionally calling `init`."""
         if "." in name:
             modname = name
         elif "external:" in name:
@@ -123,90 +119,77 @@ class Pyprland:
                     self.tasks_group.create_task(self._plugin_runner_loop(name))
             self.plugins[name] = plug
         except ModuleNotFoundError as e:
-            self.log.error("Unable to locate plugin called '%s'", name)
-            await notify_info(
-                f'Config requires plugin "{name}" but pypr can\'t find it: {e}'
-            )
+            self.log.exception("Unable to locate plugin called '%s'", name)
+            await notify_info(f'Config requires plugin "{name}" but pypr can\'t find it: {e}')
             return False
         except Exception as e:
             await notify_info(f"Error loading plugin {name}: {e}")
-            self.log.error("Error loading plugin %s:", name, exc_info=True)
-            raise PyprError() from e
+            self.log.exception("Error loading plugin %s:", name)
+            raise PyprError from e
         return True
 
-    async def __load_plugins_config(self, init=True):
-        """Loads the plugins mentioned in the config.
+    async def __load_plugins_config(self, init: bool = True) -> None:
+        """Load the plugins mentioned in the config.
+
         If init is `True`, call the `init()` method on each plugin.
         """
-
         sys.path.extend(self.config["pyprland"].get("plugins_paths", []))
 
         init_pyprland = "pyprland" not in self.plugins
 
         for name in ["pyprland"] + self.config["pyprland"]["plugins"]:
-            if name not in self.plugins:
-                if not await self._load_single_plugin(name, init):
-                    continue
+            if name not in self.plugins and not await self._load_single_plugin(name, init):
+                continue
             if init:
                 try:
                     await self.plugins[name].load_config(self.config)
                     await asyncio.wait_for(self.plugins[name].on_reload(), timeout=5.0)
-                except asyncio.TimeoutError:
+                except TimeoutError:
                     self.plugins[name].log.info("timed out on reload")
-                except PyprError:
-                    raise
                 except Exception as e:
                     await notify_info(f"Error initializing plugin {name}: {e}")
-                    self.log.error("Error initializing plugin %s:", name, exc_info=True)
-                    raise PyprError() from e
+                    self.log.exception("Error initializing plugin %s:", name)
+                    raise PyprError from e
                 else:
                     self.plugins[name].log.info("configured")
         if init_pyprland:
             plug = self.plugins["pyprland"]
             plug.set_commands(reload=self.load_config)  # type: ignore
 
-    async def load_config(self, init=True):
-        """loads the configuration (new plugins will be added & config updated)
-
-        if `init` is true, also initializes the plugins"""
+    async def load_config(self, init: bool = True) -> None:
+        """Load the configuration (new plugins will be added & config updated).
 
+        if `init` is true, also initializes the plugins
+        """
         await self.__open_config()
         assert self.config
         await self.__load_plugins_config(init=init)
         colored_logs = self.config["pyprland"].get("colored_handlers_log", True)
-        self.log_handler = (
-            self.colored_log_handler if colored_logs else self.plain_log_handler
-        )  # pylint: disable=attribute-defined-outside-init
+        self.log_handler = self.colored_log_handler if colored_logs else self.plain_log_handler
 
-    def plain_log_handler(self, plugin, name, params):
-        "log a handler method without color"
-        plugin.log.debug(f"{name}{params}")
+    def plain_log_handler(self, plugin: Plugin, name: str, params: tuple[str]) -> None:
+        """Log a handler method without color."""
+        plugin.log.debug("%s%s", name, params)
 
-    def colored_log_handler(self, plugin, name, params):
-        "log a handler method with color"
+    def colored_log_handler(self, plugin: Plugin, name: str, params: tuple[str]) -> None:
+        """Log a handler method with color."""
         color = 33 if name.startswith("run_") else 30
-        plugin.log.debug(f"\033[{color};1m%s%s\033[0m", name, params)
+        plugin.log.debug("\033[%s;1m%s%s\033[0m", color, name, params)
 
-    async def _run_plugin_handler(self, plugin, full_name, params):
-        "Runs a single handler on a plugin"
+    async def _run_plugin_handler(self, plugin: Plugin, full_name: str, params: tuple[str, ...]) -> None:
+        """Run a single handler on a plugin."""
         self.log_handler(plugin, full_name, params)
         try:
             await getattr(plugin, full_name)(*params)
         except AssertionError as e:
-            self.log.error(
-                "This could be a bug in Pyprland, if you think so, report on https://github.com/fdev31/pyprland/issues"
-            )
-            self.log.exception(e)
-            await notify_error(
-                f"Pypr integrity check failed on {plugin.name}::{full_name}: {e}"
-            )
+            self.log.exception("This could be a bug in Pyprland, if you think so, report on https://github.com/fdev31/pyprland/issues")
+            await notify_error(f"Pypr integrity check failed on {plugin.name}::{full_name}: {e}")
         except Exception as e:  # pylint: disable=W0718
-            self.log.warning("%s::%s(%s) failed:", plugin.name, full_name, params)
-            self.log.exception(e)
+            self.log.exception("%s::%s(%s) failed:", plugin.name, full_name, params)
             await notify_error(f"Pypr error {plugin.name}::{full_name}: {e}")
 
-    async def _callHandler(self, full_name, *params, notify=""):
-        "Call an event handler with params"
+    async def _call_handler(self, full_name: str, *params: str, notify: str = "") -> bool:
+        """Call an event handler with params."""
         handled = False
         for plugin in self.plugins.values():
             if hasattr(plugin, full_name):
@@ -214,23 +197,22 @@ class Pyprland:
                 task = partial(self._run_plugin_handler, plugin, full_name, params)
                 if plugin == "pyprland":
                     await task()
-                else:
-                    if not plugin.aborted:
-                        await self.queues[plugin.name].put(task)
+                elif not plugin.aborted:
+                    await self.queues[plugin.name].put(task)
         if notify and not handled:
             await notify_info(f'"{notify}" not found')
         return handled
 
-    async def read_events_loop(self):
-        "Consumes the event loop and calls corresponding handlers"
+    async def read_events_loop(self) -> None:
+        """Consume the event loop and calls corresponding handlers."""
         while not self.stopped:
             try:
                 data = (await self.event_reader.readline()).decode()
-            except RuntimeError as e:
-                self.log.error("Aborting event loop: %s", e)
+            except RuntimeError:
+                self.log.exception("Aborting event loop")
                 return
             except UnicodeDecodeError:
-                self.log.error("Invalid unicode while reading events")
+                self.log.exception("Invalid unicode while reading events")
                 continue
             if not data:
                 self.log.critical("Reader starved")
@@ -239,17 +221,17 @@ class Pyprland:
             full_name = f"event_{cmd}"
 
             # self.log.debug("[%s] %s", cmd, params.strip())
-            await self._callHandler(full_name, params.rstrip("\n"))
+            await self._call_handler(full_name, params.rstrip("\n"))
 
-    async def exit_plugins(self):
-        "Exits all plugins"
+    async def exit_plugins(self) -> None:
+        """Exit all plugins."""
         for plugin in self.plugins.values():
             if not plugin.aborted:
                 plugin.aborted = True
                 await asyncio.wait_for(plugin.exit(), timeout=2.0)
 
-    async def read_command(self, reader, writer) -> None:
-        "Receives a socket command"
+    async def read_command(self, reader: asyncio.StreamReader, writer: asyncio.StreamWriter) -> None:
+        """Receive a socket command."""
         data = (await reader.readline()).decode()
         if not data:
             self.log.critical("Server starved")
@@ -257,7 +239,7 @@ class Pyprland:
         data = data.strip()
         if data == "exit":
 
-            async def _abort():
+            async def _abort() -> None:
                 await self.exit_plugins()
                 # cancel the task group
                 for task in self.tasks:
@@ -287,18 +269,18 @@ class Pyprland:
         full_name = f"run_{cmd}"
 
         if PYPR_DEMO:
-            os.system(f"notify-send -t 4000 '{data}'")
+            os.system(f"notify-send -t 4000 '{data}'")  # noqa: ASYNC102
 
-        if not await self._callHandler(full_name, *args, notify=cmd):
+        if not await self._call_handler(full_name, *args, notify=cmd):
             self.log.warning("No such command: %s", cmd)
 
-    async def serve(self):
-        "Runs the server"
+    async def serve(self) -> None:
+        """Run the server."""
         async with self.server:
             await self.server.wait_closed()
 
-    async def _plugin_runner_loop(self, name):
-        "Runs tasks for a given plugin indefinitely"
+    async def _plugin_runner_loop(self, name: str) -> None:
+        """Run tasks for a given plugin indefinitely."""
         q = self.queues[name]
         is_pyprland = name == "pyprland"
 
@@ -311,29 +293,27 @@ class Pyprland:
                     return
                 if is_pyprland:
                     self.pyprland_mutex_event.clear()
-            except RuntimeError as e:
-                self.log.error("Aborting [%s] loop: %s", name, e)
+            except RuntimeError:
+                self.log.exception("Aborting [%s] loop", name)
                 return
             try:
                 await asyncio.wait_for(task(), timeout=12.0)
-            except asyncio.TimeoutError:
-                self.log.error("Timeout running plugin %s::%s", name, task)
-            except Exception as e:  # pylint: disable=W0718
-                self.log.error(
-                    "Unhandled error running plugin %s::%s: %s", name, task, e
-                )
+            except TimeoutError:
+                self.log.exception("Timeout running plugin %s::%s", name, task)
+            except Exception:  # pylint: disable=W0718
+                self.log.exception("Unhandled error running plugin %s::%s", name, task)
             if is_pyprland and q.empty():
                 self.pyprland_mutex_event.set()
 
-    async def plugins_runner(self):
-        "Runs plugins' task using the created `tasks` TaskGroup attribute"
+    async def plugins_runner(self) -> None:
+        """Run plugins' task using the created `tasks` TaskGroup attribute."""
         async with asyncio.TaskGroup() as group:
             self.tasks_group = group
             for name in self.plugins:
                 self.tasks.append(group.create_task(self._plugin_runner_loop(name)))
 
-    async def run(self):
-        "Runs the server and the event listener"
+    async def run(self) -> None:
+        """Run the server and the event listener."""
         await asyncio.gather(
             asyncio.create_task(self.serve()),
             asyncio.create_task(self.read_events_loop()),
@@ -341,45 +321,39 @@ class Pyprland:
         )
 
 
-async def run_daemon():
-    "Runs the server / daemon"
-    manager = Pyprland()
+async def get_event_stream_with_retry(max_retry: int = 10) -> tuple[asyncio.StreamReader, asyncio.StreamWriter] | tuple[None, Exception]:
+    """Obtain the event stream, retrying if it fails.
+
+    If retry count is exhausted, returns (None, exception).
+    """
     err_count = itertools.count()
-    manager.server = await asyncio.start_unix_server(manager.read_command, CONTROL)
-    max_retry = 10
     while True:
         attempt = next(err_count)
         try:
-            events_reader, events_writer = await get_event_stream()
+            return await get_event_stream()
         except Exception as e:  # pylint: disable=W0718
             if attempt > max_retry:
-                manager.log.critical("Failed to open hyprland event stream: %s.", e)
-                await notify_fatal("Failed to open hyprland event stream")
-                raise PyprError() from e
-            manager.log.warning(
-                "Failed to get event stream: %s, retry %s/%s...", e, attempt, max_retry
-            )
+                return None, e
             await asyncio.sleep(1)
-        else:
-            break
+
+
+async def run_daemon() -> None:
+    """Run the server / daemon."""
+    manager = Pyprland()
+    manager.server = await asyncio.start_unix_server(manager.read_command, CONTROL)
+
+    events_reader, events_writer = await get_event_stream_with_retry()
+    if events_reader is None:
+        manager.log.critical("Failed to open hyprland event stream: %s.", events_writer)
+        await notify_fatal("Failed to open hyprland event stream")
+        raise PyprError from cast(Exception, events_writer)
 
     manager.event_reader = events_reader
 
-    try:
-        await manager.initialize()
-    except PyprError as e:
-        if bool(str(e)):
-            await notify_fatal(f"Pypr failed to start: {e}")
-        else:
-            await notify_fatal("Pypr failed to start!")
-
-        raise SystemExit(1) from e
-    except Exception as e:
-        manager.log.critical("Failed to load config.", exc_info=True)
-        await notify_fatal(f"Pypr couldn't load config: {e}")
-        raise SystemExit(1) from e
+    await manager.initialize()
 
     manager.log.debug("[ initialized ]".center(80, "="))
+
     try:
         await manager.run()
     except KeyboardInterrupt:
@@ -388,32 +362,33 @@ async def run_daemon():
         manager.log.critical("cancelled")
     else:
         await manager.exit_plugins()
+        assert isinstance(events_writer, asyncio.StreamWriter)
         events_writer.close()
         await events_writer.wait_closed()
         manager.server.close()
         await manager.server.wait_closed()
 
 
-def show_help(manager):
-    "show the documentation"
+def show_help(manager: Pyprland) -> None:
+    """Show the documentation."""
 
-    def format_doc(txt):
+    def format_doc(txt: str) -> str:
         return txt.split("\n")[0]
 
     print(
         """Syntax: pypr [command]
 
-If the command is ommited, runs the daemon which will start every configured plugin.
+If the command is omitted, runs the daemon which will start every configured plugin.
 
 Available commands:
 """
     )
     builtins_docs = {
-        "dumpjson": "Dumps the configuration in JSON format",
-        "edit": "Edit the configuration file",
-        "exit": "Exits the daemon",
-        "help": "Shows this help",
-        "version": "Shows the version",
+        "dumpjson": "Dump the configuration in JSON format.",
+        "edit": "Edit the configuration file.",
+        "exit": "Exit the daemon.",
+        "help": "Show this help.",
+        "version": "Show the version.",
     }
     for name, doc in builtins_docs.items():
         print(f" {name:20s} {doc}")
@@ -428,13 +403,13 @@ Available commands:
                 print(f" {name[4:]:20s} {doc_txt} [{plug.name}]")
 
 
-async def run_client():
-    "Runs the client (CLI)"
+async def run_client() -> None:
+    """Run the client (CLI)."""
     manager = Pyprland()
 
     if sys.argv[1] == "version":
-        print("2.2.17")  # Automatically updated version
-        return
+        print(VERSION)
+        return None
 
     if sys.argv[1] == "edit":
         editor = os.environ.get("EDITOR", os.environ.get("VISUAL", "vi"))
@@ -442,7 +417,7 @@ async def run_client():
         run_interactive_program(f'{editor} "{filename}"')
         sys.argv[1] = "reload"
 
-    if sys.argv[1] in ("--help", "-h", "help"):
+    if sys.argv[1] in {"--help", "-h", "help"}:
         await manager.load_config(init=False)
         return show_help(manager)
 
@@ -450,14 +425,14 @@ async def run_client():
         await manager.load_config(init=False)
         # Dump manager.config in TOML format
         print(json.dumps(manager.config, indent=2))
-        return
+        return None
 
     try:
         _, writer = await asyncio.open_unix_connection(CONTROL)
     except (ConnectionRefusedError, FileNotFoundError) as e:
         manager.log.critical("Failed to open control socket, is pypr daemon running ?")
         await notify_error("Pypr can't connect, is daemon running ?")
-        raise PyprError() from e
+        raise PyprError from e
 
     args = sys.argv[1:]
     args[0] = args[0].replace("-", "_")
@@ -467,8 +442,9 @@ async def run_client():
     await writer.wait_closed()
 
 
-def use_param(txt):
-    """Checks if parameter `txt` is in sys.argv
+def use_param(txt: str) -> str:
+    """Check if parameter `txt` is in sys.argv.
+
     if found, removes it from sys.argv & returns the argument value
     """
     v = ""
@@ -479,8 +455,8 @@ def use_param(txt):
     return v
 
 
-def main():
-    "runs the command"
+def main() -> None:
+    """Run the command."""
     debug_flag = use_param("--debug")
     if debug_flag:
         init_logger(filename=debug_flag, force_debug=True)
