@@ -9,8 +9,8 @@
 from collections import defaultdict
 from typing import Any, cast
 
-from ..common import CastBoolMixin, state
-from ..types import ClientInfo
+from ..common import CastBoolMixin, is_rotated, state
+from ..types import ClientInfo, MonitorInfo
 from .interface import Plugin
 
 
@@ -27,12 +27,33 @@ class Extension(CastBoolMixin, Plugin):
         if not self.enabled:
             return
         win_addr = "0x" + windescr.split(",", 1)[0]
-        for i, cli in enumerate(await self.get_clients()):
-            if cli["address"] == win_addr and not cli["floating"]:
-                await self.hyprctl(f"focuswindow address:{self.main_window_addr}")
-                self.last_index = i
+
+        behavior = self.config.get("on_new_client", "focus")
+        new_client: ClientInfo | None = None
+        clients = await self.get_clients()
+        new_client_idx = 0
+        for i, cli in enumerate(clients):
+            if cli["address"] == win_addr:
+                if cli["floating"]:
+                    # Ignore floating windows
+                    return
+                new_client = cli
+                new_client_idx = i
                 break
 
+        if new_client:
+            self.last_index = new_client_idx
+            if behavior == "background":
+                # focus the main client
+                await self.hyprctl(f"focuswindow address:{self.main_window_addr}")
+            elif behavior == "close":
+                await self._run_toggle()
+            else:  # foreground
+                # make the new client the main window
+                await self.unprepare_window(clients)
+                self.main_window_addr = win_addr
+                await self.prepare_window(clients)
+
     async def event_activewindowv2(self, _: str) -> None:
         """Keep track of focused client."""
         captive = self.cast_bool(self.config.get("captive_focus"))
@@ -72,6 +93,15 @@ class Extension(CastBoolMixin, Plugin):
         else:
             await self.notify_error(f"unknown layout_center command: {what}")
 
+    async def on_reload(self) -> None:
+        """Loads the configuration and apply the tag style."""
+        if not self.config.get("style"):
+            return
+        await self.hyprctl("windowrulev2 unset, tag:layout_center")
+        commands = [f"windowrulev2 {rule}, tag:layout_center" for rule in self.config.get("style", [])]
+        if commands:
+            await self.hyprctl(commands)
+
     # Utils
 
     async def get_clients(self, *_) -> list[ClientInfo]:  # pylint: disable=arguments-differ
@@ -88,6 +118,9 @@ class Extension(CastBoolMixin, Plugin):
         for cli in clients:
             if cli["address"] == addr and cli["floating"]:
                 await self.hyprctl(f"togglefloating address:{addr}")
+                if self.config.get("style"):
+                    await self.hyprctl(f"tagwindow -layout_center address:{addr}")
+                break
 
     async def prepare_window(self, clients: list[ClientInfo] | None = None) -> None:
         """Set the window as centered."""
@@ -97,18 +130,24 @@ class Extension(CastBoolMixin, Plugin):
         for cli in clients:
             if cli["address"] == addr and not cli["floating"]:
                 await self.hyprctl(f"togglefloating address:{addr}")
+                if self.config.get("style"):
+                    await self.hyprctl(f"tagwindow +layout_center address:{addr}")
+                break
         width = 100
         height = 100
         x, y = self.offset
-        margin = self.margin
+        m = self.margin
+        margin: tuple[int, int] = (m, m) if isinstance(m, int) else m
         scale = 1
         for monitor in cast(list[dict[str, Any]], await self.hyprctl_json("monitors")):
             scale = monitor["scale"]
             if monitor["focused"]:
-                width = monitor["width"] - (2 * margin)
-                height = monitor["height"] - (2 * margin)
-                x += monitor["x"] + margin
-                y += monitor["y"] + margin
+                width = monitor["width"] - (2 * margin[0])
+                height = monitor["height"] - (2 * margin[1])
+                if is_rotated(cast(MonitorInfo, monitor)):
+                    width, height = height, width
+                x += monitor["x"] + margin[0]
+                y += monitor["y"] + margin[1]
                 break
         await self.hyprctl(f"resizewindowpixel exact {int(width / scale)} {int(height / scale)},address:{addr}")
         await self.hyprctl(f"movewindowpixel exact {int(x / scale)} {int(y / scale)},address:{addr}")
